@@ -1,5 +1,4 @@
 from copy import copy, deepcopy
-import engpy.tools.exprs as exprs
 from engpy.AI import _math
 from engpy.oblects.abc import Utilities, UtilityClass
 from engpy.oblects.abc import BaseClass, ExpressionObjectClass
@@ -8,16 +7,18 @@ from engpy.misc.miscs import num, alnum, counternum
 from engpy.misc.gen import con, rev, reverse, getter
 from engpy.misc.assist import cd2str, d2lst, join, arrange_, m_char
 from engpy.misc.assist import copy, deepcopy, get_exprs
-from engpy.misc.miscs import simp_var
+from engpy.misc.miscs import simp_var, lexer
 from engpy.misc.vars import alpha
 from engpy.misc.abilities import numable, intable
 from engpy.errors.exceptions import *
 import math as mth
 
 e = 2.718281828459045
+
+
 class Log(BaseClass):
 
-    def __init__(self, arg=0, base=10, coeff=1, hkeys = '', s=False):
+    def __init__(self, arg=0, base=10, coeff=1, hkeys='', s=False):
         self.s = s
         self.name = 'Log'
         if isinstance(arg, dict):
@@ -25,34 +26,36 @@ class Log(BaseClass):
             return
         self.base = alnum(base) if not s else 'e'
         self.coeff = alnum(coeff)
-        self.arg = exprs.Expr(f"{arg}", hkeys = hkeys)
+        self.arg = lexer(f"{arg}", hkeys=hkeys)
 
     def __str__(self):
         base = ('ln' if isinstance(self.base,
                                    float) and round(self.base,
                                                     10) == round(e,
                                                                  10) else 'log' if self.base == 10 else f"log{self.base}")
-        arg = f"({self.arg})" if format(self.arg)[0] != '(' or  format(self.arg)[-1] != ')'  else f"{self.arg}"
+        arg = f"({self.arg})" if format(self.arg)[0] != '(' or format(self.arg)[-1] != ')' else f"{self.arg}"
         coeff = '' if self.coeff == 1 else self.coeff
-        return f"{str(coeff)}{base}{arg}"     
+        return f"{str(coeff)}{base}{arg}"
 
     def diff(self, var):
-        return ((self.coeff*self.arg.lin_diff(var))/self.arg * (1 if isinstance(self.base,
-                                   float) and round(self.base,
-                                                    10) == round(alnum(e),
-                                                                 10) else exprs.Expr(f'log{self.base}(.e)')))
+        return ((self.coeff * self.arg.lin_diff(var)) / self.arg * (1 if isinstance(self.base,
+                                                                                    float) and round(self.base,
+                                                                                                     10) == round(
+            alnum(e),
+            10) else lexer(f'log{self.base}(.e)')))
 
-    def cal(self, values = '', desolve = True):
-        arg = self.arg.cal(values,desolve = desolve) if getter(self.arg,'name') else self.arg
-        coeff = self.coeff.cal(values,desolve = desolve) if getter(self.coeff,'name') else self.coeff
-        base = self.base.cal(values,desolve = desolve) if getter(self.base,'name') else self.base
+    def cal(self, values='', desolve=True):
+        arg = self.arg.cal(values, desolve=desolve) if getter(self.arg, 'name') else self.arg
+        coeff = self.coeff.cal(values, desolve=desolve) if getter(self.coeff, 'name') else self.coeff
+        base = self.base.cal(values, desolve=desolve) if getter(self.base, 'name') else self.base
         try:
-            return coeff * exprs.Expr(str(mth.log10(arg)/mth.log10(base)))
+            return coeff * lexer(str(mth.log10(arg) / mth.log10(base)))
         except ValueError:
-            raise InvalidOperation(f"log to base 10 is only valid for positive numbers: log10({arg if arg <= 0 else base})")
+            raise InvalidOperation(
+                f"log to base 10 is only valid for positive numbers: log10({arg if arg <= 0 else base})")
         except InvalidOperation:
             return log(f'{coeff}log{base}({arg})')
-    
+
     @property
     def vars(self):
         var_list = []
@@ -79,15 +82,13 @@ class Log(BaseClass):
     def __nq__(self, other):
         return self.__hash__() != other.__hash__()
 
-    
-
     __repr__ = __str__
 
 
 class log(ExpressionObjectClass, UtilityClass):
 
-    def __init__(self, expr, hkeys = '', skip=False):
-        
+    def __init__(self, expr, hkeys='', skip=False):
+
         self.name = 'log'
         if skip or isinstance(expr, dict):
             self.expr = expr
@@ -95,13 +96,13 @@ class log(ExpressionObjectClass, UtilityClass):
         expr = str(expr)
         expr = expr.replace(' ',
                             '').replace('##',
-                                        '').replace('^-','??')
+                                        '').replace('^-', '??')
         keys = {}
         while '(' in expr:
             keys_ = m_char('#', len(keys) + 3)
             keys[keys_] = get_exprs(expr, expr.index('('))[0]
             expr = expr.replace(keys[keys_], keys_)
-            
+
         expr = expr.replace('ln', 'loge')
         if '+' in expr:
             part = expr.split('+')
@@ -152,15 +153,18 @@ class log(ExpressionObjectClass, UtilityClass):
         expr = expr.replace(';', '-').replace('%%%',
                                               '(-').replace('%%',
                                                             'n-').replace('%',
-                                                                          's-').replace('??','^-')
+                                                                          's-').replace('??', '^-')
         for key, value in reverse(keys).items():
             expr = expr.replace(key, value)
-        n = 0;coeff = '';expr__ = {}
+        n = 0
+        coeff = ''
+        expr__ = {}
         while n < len(expr):
             if expr[n] in 'l':
                 u = expr[n:n + 3] if (not expr[(n + 1)] == 'n') else (expr[n:n + 2])
                 n += 3 if (not expr[(n + 1)] == 'n') else 2
-                args = ''; pow_ = ''
+                args = ''
+                pow_ = ''
                 while True:
                     if expr[n] == '(':
                         break
@@ -171,16 +175,17 @@ class log(ExpressionObjectClass, UtilityClass):
                     track = 0
                     if expr[n] == '(':
                         args, n = get_exprs(expr, n)
-                        args = args[1:-1]; pow_ = pow_.replace('e','.e')
+                        args = args[1:-1]
+                        pow_ = pow_.replace('e', '.e')
                         pow_ = '10' if not pow_ else '-1' if pow_ == '-' else pow_
-                        
+
                         pow_ = alnum(pow_)
                         _pow_ = ''
-                        
+
                         if n < len(expr) and expr[n] == '^':
                             n += 1
                             while n < len(expr):
-                                    
+
                                 if expr[n] == 'l':
                                     n -= 1
                                     break
@@ -214,17 +219,25 @@ class log(ExpressionObjectClass, UtilityClass):
             coeff_ = coeff
             coeff = counternum(coeff)
             if coeff < 0:
-                disp += (f" - {abs(num(coeff))}" if numable(coeff_) else (f"{coeff_}")) if coeff_ != -1 else ' - 1' if self.expr[coeff_] == [{'': 0}] else ' - '
+                disp += (f" - {abs(num(coeff))}" if numable(coeff_) else f"{coeff_}") if coeff_ != -1 else ' - 1' if \
+                    self.expr[coeff_] == [{'': 0}] else ' - '
             else:
                 if disp:
                     disp = ' + '
-                disp += (f"{coeff_}") if coeff_ != 1 else ' 1 ' if (coeff_ == 1) and (self.expr[coeff_] == [{'': 0}]) else ''
+                disp += f"{coeff_}" if coeff_ != 1 else ' 1 ' if (coeff_ == 1) and (
+                        self.expr[coeff_] == [{'': 0}]) else ''
             for count, expr_ in enumerate(self.expr[coeff]):
                 if count:
-                    disp += ' - 1 ' if ((f"{coeff_}") if coeff < 0 else ' + {coeff_}' if coeff > 0 else coeff == -1) and (self.expr[coeff_] == [{'': 0}] or self.expr[coeff_][count] == {'': 0}) else ' + 1' if (coeff == 1) and (self.expr[coeff] == [{'': 0}] or self.expr[coeff][count] == {'': 0}) else ' - ' if coeff == -1 else f" - {abs(num(coeff))}" if numable(coeff_) else f" - {coeff}" if coeff < 0 else f" + {coeff}" if coeff > 0 else ' + '
+                    disp += ' - 1 ' if ((
+                                            f"{coeff_}") if coeff < 0 else ' + {coeff_}' if coeff > 0 else coeff == -1) and (
+                                               self.expr[coeff_] == [{'': 0}] or self.expr[coeff_][count] == {
+                                           '': 0}) else ' + 1' if (coeff == 1) and (
+                            self.expr[coeff] == [{'': 0}] or self.expr[coeff][count] == {
+                        '': 0}) else ' - ' if coeff == -1 else f" - {abs(num(coeff))}" if numable(
+                        coeff_) else f" - {coeff}" if coeff < 0 else f" + {coeff}" if coeff > 0 else ' + '
                 for var in expr_:
-                    disp += (f"{var}") if var else str(abs(coeff)) if (not disp) else ''
-                    if expr_[var] not in (1,0):
+                    disp += f"{var}" if var else str(abs(coeff)) if (not disp) else ''
+                    if expr_[var] not in (1, 0):
                         disp += f"^{expr_[var]}"
                 if stray:
                     disp += '#'
@@ -234,7 +247,7 @@ class log(ExpressionObjectClass, UtilityClass):
     def __add__(self, other):
         self_ = deepcopy(self)
         if isinstance(other, (str, float, int)):
-            other = log((f"{other}"))
+            other = log(f"{other}")
         if len(other) > 1:
             for expr in other.struct:
                 self_ += expr
@@ -249,7 +262,7 @@ class log(ExpressionObjectClass, UtilityClass):
     def __sub__(self, other):
         self_ = copy(self)
         if isinstance(other, (str, float, int)):
-            other = log((f"{other}"))
+            other = log(f"{other}")
         if len(other) > 1:
             for expr in other:
                 self_ -= expr
@@ -280,28 +293,41 @@ class log(ExpressionObjectClass, UtilityClass):
             coeff_ = coeff
             coeff = counternum(coeff)
             if coeff < 0:
-                disp += (f" - {abs(num(coeff))}" if numable(coeff_) else (f"{coeff_}")) if coeff_ != -1 else ' - 1' if self.expr[coeff_] == [{'': 0}] else ' - '
+                disp += (f" - {abs(num(coeff))}" if numable(coeff_) else f"{coeff_}") if coeff_ != -1 else ' - 1' if \
+                    self.expr[coeff_] == [{'': 0}] else ' - '
             else:
                 if disp:
                     disp = ' + '
-                disp = f'{coeff_}' if coeff < 0 else ' + {coeff_}' if coeff > 0 else ' - 1 ' if coeff == -1 and (self.expr[coeff_] == [{'':0}] or self.expr[coeff_][count] == {'':0}) else ' + 1' if coeff ==  1 and (self.expr[coeff] == [{'': 0}] or self.expr[coeff][count] == {'':0}) else f' - ' if coeff == -1 else (f' - {abs(num(coeff))}' if numable(coeff_) else f'{coeff}') if coeff < 0 else ' + ' if coeff == 1 else f' + {coeff}' if coeff > 0 else ' + '
+                disp = f'{coeff_}' if coeff < 0 else ' + {coeff_}' if coeff > 0 else ' - 1 ' if coeff == -1 and (
+                        self.expr[coeff_] == [{'': 0}] or self.expr[coeff_][count] == {
+                    '': 0}) else ' + 1' if coeff == 1 and (
+                        self.expr[coeff] == [{'': 0}] or self.expr[coeff][count] == {
+                    '': 0}) else f' - ' if coeff == -1 else (f' - {abs(num(coeff))}' if numable(
+                    coeff_) else f'{coeff}') if coeff < 0 else ' + ' if coeff == 1 else f' + {coeff}' if coeff > 0 else ' + '
             for count, expr_ in enumerate(self.expr[coeff]):
                 if count:
-                    disp = ' - 1 ' if ((f"{coeff_}") if coeff < 0 else ' + {coeff_}' if coeff > 0 else coeff == -1) and (self.expr[coeff_] == [{'': 0}] or self.expr[coeff_][count] == {'': 0}) else ' + 1' if (coeff == 1) and (self.expr[coeff] == [{'': 0}] or self.expr[coeff][count] == {'': 0}) else ' + ' if coeff == 1 else ' - ' if coeff == -1 else (f" - {abs(num(coeff))}" if numable(coeff_) else (f" - {abs(num(coeff))}") if coeff < 0 else f" + {coeff}" if coeff > 0 else ' + ')
+                    disp = ' - 1 ' if ((
+                                           f"{coeff_}") if coeff < 0 else ' + {coeff_}' if coeff > 0 else coeff == -1) and (
+                                              self.expr[coeff_] == [{'': 0}] or self.expr[coeff_][count] == {
+                                          '': 0}) else ' + 1' if (coeff == 1) and (
+                            self.expr[coeff] == [{'': 0}] or self.expr[coeff][count] == {
+                        '': 0}) else ' + ' if coeff == 1 else ' - ' if coeff == -1 else (
+                        f" - {abs(num(coeff))}" if numable(coeff_) else (
+                            f" - {abs(num(coeff))}") if coeff < 0 else f" + {coeff}" if coeff > 0 else ' + ')
                 for var in expr_:
-                    disp += (f"{var}") if var else str(abs(coeff)) if (not disp) else ''
-                    if expr_[var] not in (1,0):
-                            disp += f"^{expr_[var]}"
+                    disp += f"{var}" if var else str(abs(coeff)) if (not disp) else ''
+                    if expr_[var] not in (1, 0):
+                        disp += f"^{expr_[var]}"
                 yield disp
 
     def __mul__(self, other):
         if isinstance(other, (int, float)):
-            other = exprs.Expr(str(other))
+            other = lexer(str(other))
         elif isinstance(other, (str, Log)):
             other = log(other)
         elif getter(other, 'name') not in ('trig', 'Expr', 'log'):
             raise KindError(f"{type(other)} is not supported")
-        sef_ = {};val_ = {};_val_= []; statr = 0
+        sef_, val_, _val_, statr = {}, {}, [], 0
         other = other.simp()
         if not other or not self:
             return log(0)
@@ -316,7 +342,7 @@ class log(ExpressionObjectClass, UtilityClass):
                         for _vars__ in vars__:
                             if _vars__ in _var:
                                 val_[_vars__] = vars__[_vars__] + _var[_vars__]
-                                
+
                             else:
                                 val_[_vars__] = vars__[_vars__]
                             for __vars__ in _var:
@@ -325,8 +351,9 @@ class log(ExpressionObjectClass, UtilityClass):
                         for item in copy(val_):
                             if not val_[item]:
                                 val_.pop(item)
-                                
-                        _val_.append(val_);val_ = {}
+
+                        _val_.append(val_)
+                        val_ = {}
 
                     _sef_.expr[alnum(_coeff)] = _val_
                     _val_ = []
@@ -345,15 +372,16 @@ class log(ExpressionObjectClass, UtilityClass):
     def __hash__(self):
         return self.__str__().__hash__()
 
-    def lin_diff(self, var = ''):
+    def lin_diff(self, var=''):
         if not var:
             var = _math['working var']
-        _log = exprs.Expr({})
+        _log = lexer({})
         for terms in self.struct:
             if isinstance(terms, log):
-                coeff = terms.coeff; log_expr = list(terms.expr[coeff][0])[0]
+                coeff = terms.coeff
+                log_expr = list(terms.expr[coeff][0])[0]
                 pow_ = terms.expr[coeff][0][log_expr]
-                log_expr.coeff = coeff * pow_ 
+                log_expr.coeff = coeff * pow_
                 _log += log_expr.diff(var)
         return _log.simp()
 
@@ -374,7 +402,7 @@ class log(ExpressionObjectClass, UtilityClass):
 
     def simp(self):
         if self.expr in ({0: [{'': 0}]}, {0: [{}]}):
-            return trig(0)
+            return log(0)
         res = {}
         for coeff, var in self.expr.items():
             if not coeff:
@@ -410,10 +438,10 @@ class log(ExpressionObjectClass, UtilityClass):
     def __bool__(self):
         return self.expr != {0: [{'': 0}]}
 
-    def cal(self, value='', desolve = True, **values):
+    def cal(self, value='', desolve=True, **values):
         var_list = self.vars
         if isinstance(value, (float, int, str)) and not values and var_list != 1:
-            raise BadRequest('Operation not Understood')
+            raise Vague('Operation not Understood')
         if isinstance(value, (float, int, str)) and not values and var_list == 1:
             value = {var_list[0]: value}
         if isinstance(value, dict):
@@ -433,14 +461,15 @@ class log(ExpressionObjectClass, UtilityClass):
                 coeffs = log(coeffs if numable(coeffs) else coeffs.cal(values))
                 for vars__ in vars_[0]:
                     if isinstance(vars__, Log):
-                        _vars = vars__.cal(values, desolve = desolve) ** (vars_[0][vars__] if numable(vars_[0][vars__]) else vars_[0][vars__].cal(values, desolve = desolve))
+                        _vars = vars__.cal(values, desolve=desolve) ** (
+                            vars_[0][vars__] if numable(vars_[0][vars__]) else vars_[0][vars__].cal(values,
+                                                                                                    desolve=desolve))
                     else:
                         _vars = 1
                     coeffs *= _vars
 
             stat += coeffs
 
-        values = {}
         return stat.simp()
 
     def __gt__(self, other):
@@ -468,7 +497,6 @@ class log(ExpressionObjectClass, UtilityClass):
         def __iter__(self):
             return self
 
-
     @property
     def varst(self):
         var = []
@@ -484,7 +512,7 @@ class log(ExpressionObjectClass, UtilityClass):
 
     def __truediv__(self, other):
         if numable(self):
-            return exprs.Expr(self) / other
+            return lexer(self) / other
         if other.__repr__() == '0':
             raise ZeroDivisionError('Division by Zero')
         val = {}
@@ -525,7 +553,7 @@ class log(ExpressionObjectClass, UtilityClass):
             return log({1: [{self: other}]})
         for interger in range(abs(other) - 1):
             self_ *= self
-        return self_ if other > 0 else Expr('1') if not other else 1/self_
+        return self_ if other > 0 else lexer('1') if not other else 1 / self_
 
     @property
     def coeff(self):
@@ -535,7 +563,7 @@ class log(ExpressionObjectClass, UtilityClass):
 
     def __rtruediv__(self, other):
         if numable(self):
-            return other / exprs.Expr(self)
+            return other / lexer(self)
         self_ = deepcopy(self)
         if other == 1:
             if len(self) != 1:
@@ -549,8 +577,8 @@ class log(ExpressionObjectClass, UtilityClass):
         return self_
 
     def __round__(self, fix):
-        coeff = [round(coeff,fix) for coeff in self.expr]
+        coeff = [round(coeff, fix) for coeff in self.expr]
         vars_ = list(self.expr.values())
-        return log(dict(zip(coeff,vars_)))
+        return log(dict(zip(coeff, vars_)))
 
     __repr__ = __str__
